/*
    Copyright (C) 2000 Paul Barton-Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: filebuffer.c,v 1.2 2000/10/29 04:42:51 pbd Exp $
*/

#define __USE_UNIX98  /* to get pread, pwrite */
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <limits.h>
#include <errno.h>

#include "seq.h"
#include "filebuffer.h"

static BufferType
file_buffer_type (buffer_t *buf)

{
	return FileBuffer;
}

static gchar *
file_buffer_id (buffer_t *buf)

{
	file_buffer_t *fb = (file_buffer_t *) buf->private_data;
        return g_strdup (fb->name); /* XXX leak like a sieve */
}

static guint32
file_buffer_length (buffer_t *buf)

{
	return ((file_buffer_t *) buf->private_data)->length;
}

static guint32
file_buffer_read (buffer_t *src, buffer_data_t *dst, guint32 start, guint32 cnt) 
{
	file_buffer_t *fb = (file_buffer_t *) src->private_data;

	if (pread (fb->fd, (char *) dst, 
		   cnt * sizeof (buffer_data_t), 
		   fb->offset + (start * sizeof (buffer_data_t))) < 0) {
		return 0;
	} else {
		return cnt;
	}
}

guint32
file_buffer_append (buffer_t *dst, buffer_t *src, guint32 start, guint32 cnt)
{
	file_buffer_t *dst_fb = (file_buffer_t *) dst->private_data;
	file_buffer_t *src_fb = (file_buffer_t *) src->private_data;
	buffer_data_t *copy_buf;
	int ret;
	guint32 byte_cnt = cnt * sizeof (buffer_data_t);

	copy_buf = g_malloc (byte_cnt);

	if ((ret = pread (src_fb->fd, copy_buf, 
			  byte_cnt, 
			  src_fb->offset + (start * sizeof (buffer_data_t))) != (off_t) cnt)) {
		goto out;
	}

	if ((ret = pwrite (dst_fb->fd, copy_buf, 
			   byte_cnt, 
			   dst_fb->offset + (dst_fb->length * sizeof (buffer_data_t))) != (off_t) cnt)) {
		goto out;
	}
	
	dst_fb->length += cnt;

  out:
	g_free (copy_buf);
	return ret;
}

static piece_t *
file_buffer_append_data (buffer_t *dst, buffer_data_t *data, guint32 cnt)
{
	file_buffer_t *dst_fb = (file_buffer_t *) dst->private_data;
	piece_t *p = NULL;
	size_t byte_cnt = cnt * sizeof (buffer_data_t);
	
	if (pwrite (dst_fb->fd, data, byte_cnt,
		    dst_fb->offset + (dst_fb->length * sizeof (buffer_data_t))) == (off_t) byte_cnt) {
		p = piece_new (dst, dst_fb->length, cnt);
		dst_fb->length += cnt;
	}
	return p;
}

static buffer_t *_file_buffer_new (const gchar *path, int rw);

buffer_t *
file_buffer_clone (buffer_t *orig)

{
	file_buffer_t *fb = (file_buffer_t *) orig->private_data;
	char new_name[PATH_MAX+1];

	/* XXX This is wrong, because we can end up having two file
	   buffer's using the same add file, at which point
	   append_data() will fail to do its job. Think hard about
	   this.
	*/

	if (fb->add_buffer == NULL) {
		sprintf (new_name, "%s.add", fb->name);
		fb->add_buffer = _file_buffer_new (new_name, 1);
	}

	return fb->add_buffer;
}
	
static void
file_buffer_close (buffer_t *buf)

{
	file_buffer_t *fb = (file_buffer_t *) buf->private_data;

	close (fb->fd);
	g_free (fb->name);
	g_free (fb);
}

static buffer_t *
_file_buffer_new (const gchar *path, gint add)

{
	int fd;
	struct stat statbuf;
	file_buffer_t *fb;

	if ((fd = open (path, O_RDWR|O_CREAT, 0644)) < 0) {
		return NULL;
	}
	
	if (fstat (fd, &statbuf)) {
		close (fd);
		return NULL;
	}
	
	fb = (file_buffer_t *) g_malloc (sizeof (file_buffer_t));
	fb->fd = fd;
	fb->name = g_strdup (path);
	fb->add_buffer = NULL;

	/* Add buffers start at the current end of the file.
	   XXX This is all wrong. Think about it.
	*/

	if (add) {
		fb->offset = statbuf.st_size;
	} else {
		fb->offset = 0;
	}

	fb->length = (statbuf.st_size - fb->offset) / sizeof (buffer_data_t);

	return buffer_new (file_buffer_type,
			   file_buffer_id,
			   file_buffer_length,
			   file_buffer_read,
			   file_buffer_clone,
			   file_buffer_append,
			   file_buffer_append_data,
			   file_buffer_close,
			   (gpointer) fb);
}

buffer_t *
file_buffer_new (const gchar *path)

{
	return _file_buffer_new (path, 0);
}
