;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;  config-defs.scm 
;;;  functions/classes needed in basic configuration files.  
;;;
;;;  Bill Gribble <grib@billgribble.com> 2/21/2000
;;;  $Id: config-defs.scm,v 1.2 2000/04/26 05:17:51 pbd Exp $
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(depend 'simple-obj)
(depend 'enumeration)
(depend 'soundcard)
(depend 'mididev)
(support 'config-defs)

(define <config-variable> 
  (make-simple-class 'config-variable
                     '(tag-string type where default)))

(define (config-variable? obj)
  (eq? (simple-obj-type obj) 'config-variable))

(define (config-variable:tag-string self)
  (simple-obj-getter self <config-variable> 'tag-string))

(define (config-variable:set-tag-string! self value)
  (simple-obj-setter self <config-variable> 'tag-string value))

(define (config-variable:type self)
  (simple-obj-getter self <config-variable> 'type))

(define (config-variable:set-type! self value)
  (simple-obj-setter self <config-variable> 'type value))

(define (config-variable:where self)
  (simple-obj-getter self <config-variable> 'where))

(define (config-variable:set-where! self value)
  (simple-obj-setter self <config-variable> 'where value))

(define (make-config-variable tag-string type where . default)
  (let ((val 
         (make-simple-obj <config-variable>)))
    
    (config-variable:set-tag-string! val tag-string)
    (config-variable:set-type! val type)
    (config-variable:set-where! val where)
    
    (if (pair? default)
        (set-config-var val (car default)))
    val))


;;  (set-config-var var value)
;;  the main user interface to the config variable system.
;;
;;  var must be of type config-variable, previously created by 
;;  make-config-variable.  
;;  
;;  value must be of whatever type the config-variable-type field 
;;  of var specifies.  Otherwise an error is thrown. 
;;
;;  the idea is that we check the Scheme value to make sure it's 
;;  of the appropriate type, then pass it off to a private glue 
;;  function which ends up associating the tag-string of the 
;;  variable with the value on the c side. 

(define (set-config-var var value)
  (define (type-mismatch-error)
    (error "set-config-var: variable" (config-variable:tag-string var)
           "requires value of type" (config-variable:type var)
           "but you assigned" value))
  
  (if (not (config-variable? var))
      (error "set-config-var: not a configuration variable."))
  
  ;; switch on the type specified by the variable definition
  (cond 
   ;; integer config variables 
   ((eq? (config-variable:type var) 'integer)
    (if (exact? value)
        (%set-integer-c-config-variable (config-variable:tag-string var)
                                        value)
        (type-mismatch-error)))
   
   ;; real (IEEE 32-bit float) variables
   ((eq? (config-variable:type var) 'real)
    (if (number? value)
        (%set-real-c-config-variable (config-variable:tag-string var)
                                     value)
        (type-mismatch-error)))
   
   ;; boolean variables
   ((eq? (config-variable:type var) 'boolean)
    (if (boolean? value)        
        (%set-boolean-c-config-variable (config-variable:tag-string var)
                                        value)
        (if (or (eq? 'true value) (eq? 'false value)
                (eq? 'yes value) (eq? 'no value))
            (%set-boolean-c-config-variable (config-variable:tag-string var)
                                            (if (or (eq? 'true value) 
                                                    (eq? 'yes value))
                                                #t #f))
            (type-mismatch-error))))
   
   ;; sound card 
   ((eq? (config-variable:type var) 'soundcard)
    (if (soundcard? value)
        (begin 
          (%set-soundcard-c-config-variable (config-variable:tag-string var)
                                            value))
        (type-mismatch-error)))
   
   ;; MIDI device
   ((eq? (config-variable:type var) 'mididev)
    (if (mididev? value)
        (%set-mididev-c-config-variable (config-variable:tag-string var)
                                        value)
        (type-mismatch-error)))
   
   ;; string (c++, not a c char * type string)
   ((eq? (config-variable:type var) 'string)
    (if (string? value)
        (%set-string-c-config-variable (config-variable:tag-string var)
                                       value)
        (type-mismatch-error)))
   
   ;; string (c char * type string)
   ((eq? (config-variable:type var) 'charptr)
    (if (string? value)
        (%set-charptr-c-config-variable (config-variable:tag-string var)
                                       value)
        (type-mismatch-error)))
   
   ;; list (of strings)
   ((eq? (config-variable:type var) 'charptrlist)
    (if (list? value)
        (%set-charptrlist-c-config-variable (config-variable:tag-string var)
                                            value)
        (if (string? value)
            (%set-charptrlist-c-config-variable 
             (config-variable:tag-string var)
             (list value))
            (type-mismatch-error))))
   
     ;; list (of soundcard defs)
;;   ((eq? (config-variable:type var) 'soundcardlist)
;;    (if (list? value)
;;        (%set-soundcardlist-c-config-variable (config-variable:tag-string var)
;;                                              value)
;;        (if (soundcard? value)
;;            (%set-soundcardlist-c-config-variable 
;;             (config-variable:tag-string var)
;;             (list value))
;;            (type-mismatch-error))))
   
   ;; list (of mididev defs)
   ((eq? (config-variable:type var) 'mididevlist)
    (if (list? value)
        (%set-mididevlist-c-config-variable (config-variable:tag-string var)
                                              value)
        (if (mididev? value)
            (%set-mididevlist-c-config-variable 
             (config-variable:tag-string var)
             (list value))
            (type-mismatch-error))))
   
   ;; enumeration 
   ((enumeration? (config-variable:type var))
    (if (enumeration:symbol-to-value (config-variable:type var) value)
        (%set-integer-c-config-variable 
         (config-variable:tag-string var) 
         (enumeration:symbol-to-value (config-variable:type var) value))
        (type-mismatch-error)))
   
   ;; default
   (#t
    (error "set-config-var: variable does not have a known type."))))


;;  (get-config-var var value)
;;  fetch the value from the C side and set it up as a Scheme 
;;  value. 

(define (get-config-var var)
  (if (not (config-variable? var))
      (error "get-config-var: argument is not a config-variable."))
  
  (let ((thunk-alist
         (list (cons 'integer %get-integer-c-config-variable)
               (cons 'real %get-real-c-config-variable)
               (cons 'boolean %get-boolean-c-config-variable)
               (cons 'soundcard %get-soundcard-c-config-variable)
               (cons 'mididev %get-mididev-c-config-variable)           
               (cons 'string %get-string-c-config-variable)
               (cons 'charptr  %get-charptr-c-config-variable)
               (cons 'charptrlist  %get-charptrlist-c-config-variable)
               (cons 'mididevlist %get-mididevlist-c-config-variable)
;           (soundcardlist . %get-soundcardlist-c-config-variable)
               (cons 'enum  %get-integer-c-config-variable)))
        (type-symbol #f)
        (c-value #f))
    
    (if (enumeration? (config-variable:type var))
        (set! type-symbol 'enum)
        (set! type-symbol (config-variable:type var)))
    (set! c-value ((cdr (assq type-symbol thunk-alist))
                   (config-variable:tag-string var)))
    (if (enumeration? (config-variable:type var))
        (enumeration:value-to-symbol (config-variable:type var) c-value)
        c-value)))









